// datetime 工具函数
import { pad, transDateStr } from '../../../tunas'
import { DateColumn } from './interface'

export function isDateTime (str) {
  str = transDateStr(str)
  const stamp: number = +new Date(str)
  return !isNaN(stamp)
}

export function isTime (str) {
  return /^([01][0-9]|2[0-3]):([012345][0-9])$/.test(str)
}

export function isHour (str) {
  return /^(0|[1-9]|1[0-9]|2[0-3])?$/.test(str)
}

export function genYears (config: any): DateColumn[] {
  const years: DateColumn[] = []
  const now = new Date()
  let start: number = now.getFullYear() - 10
  let end: number = now.getFullYear() + 10

  if (config.startDate) {
    start = new Date(transDateStr(config.startDate)).getFullYear()
  }
  if (config.endDate) {
    end = new Date(transDateStr(config.endDate)).getFullYear()
  }

  while (start <= end) {
    let s: string = String(start)
    years.push({ value: s, name: config.format.replace('{value}', s) })
    start++
  }

  return years
}

export function genMonths (config: any): DateColumn[] {
  const months: DateColumn[] = []
  let start: number = 1
  let end: number = 12

  if (config.startDate) {
    const startDate = new Date(transDateStr(config.startDate))
    if (startDate.getFullYear() === +config.currentYear) {
      start = startDate.getMonth() + 1
    }
  }
  if (config.endDate) {
    const endDate = new Date(transDateStr(config.endDate))
    if (endDate.getFullYear() === +config.currentYear) {
      end = endDate.getMonth() + 1
    }
  }

  while (start <= end) {
    let t = String(pad(start))
    months.push({ value: t, name: config.format.replace('{value}', t) })
    start++
  }

  return months
}

export function genDays (config: any): DateColumn[] {
  const days: DateColumn[] = []
  const now = new Date()
  let year: number = now.getFullYear()
  let month: number = now.getMonth()

  if (config.currentYear) year = +config.currentYear
  if (config.currentMonth) month = +config.currentMonth - 1

  let end = 30
  if ([0, 2, 4, 6, 7, 9, 11].indexOf(month) > -1) {
    end = 31
  } else if (month === 1) {
    end = year % 100 === 0 ? (year % 400 === 0 ? 29 : 28) : (year % 4 === 0 ? 29 : 28)
  }
  if (config.endDate) {
    const endDate = new Date(transDateStr(config.endDate))
    if (endDate.getMonth() + 1 === +config.currentMonth &&
      endDate.getFullYear() === +config.currentYear && endDate.getDate() < end
    ) {
      end = endDate.getDate()
    }
  }

  let d = 1
  if (config.startDate) {
    const startDate = new Date(transDateStr(config.startDate))
    if (startDate.getMonth() + 1 === +config.currentMonth &&
      startDate.getFullYear() === +config.currentYear
    ) {
      d = startDate.getDate()
    }
  }

  while (d <= end) {
    let t = String(pad(d))
    days.push({ value: t, name: config.format.replace('{value}', t) })
    d++
  }

  return days
}

export function genHours (config) {
  const hours: DateColumn[] = []
  const startHour = +config.startHour
  const endHour = +config.endHour
  let start = startHour
  let end = endHour

  if (end < start) {
    end = 23
  }

  if (config.startDate) {
    const startDate = new Date(transDateStr(config.startDate))
    if (startDate.getFullYear() === +config.currentYear &&
      startDate.getMonth() + 1 === +config.currentMonth &&
      startDate.getDate() === +config.currentDay && start <= startHour
    ) {
      start = startDate.getHours()
      if (start < startHour) {
        start = startHour
      }
    }
  }
  if (config.endDate) {
    const endDate = new Date(transDateStr(config.endDate))
    if (endDate.getFullYear() === +config.currentYear &&
      endDate.getMonth() + 1 === +config.currentMonth &&
      endDate.getDate() === +config.currentDay
    ) {
      end = endDate.getHours()
    }
    if (end > endHour) {
      end = endHour
    }
  }

  while (start <= end) {
    let t = String(pad(start))
    hours.push({ value: t, name: config.format.replace('{value}', t) })
    start++
  }

  return hours
}

export function genMinutes (config) {
  const minutes: DateColumn[] = []
  let start = 0
  let end = 59

  if (config.startDate) {
    const startDate = new Date(transDateStr(config.startDate))
    if (startDate.getFullYear() === +config.currentYear &&
      startDate.getMonth() + 1 === +config.currentMonth &&
      startDate.getDate() === +config.currentDay &&
      startDate.getHours() === +config.currentHour
    ) {
      start = startDate.getMinutes()
    }
  }
  if (config.endDate) {
    const endDate = new Date(transDateStr(config.endDate))
    if (endDate.getFullYear() === +config.currentYear &&
      endDate.getMonth() + 1 === +config.currentMonth &&
      endDate.getDate() === +config.currentDay &&
      endDate.getHours() === +config.currentHour
    ) {
      end = endDate.getMinutes()
    }
  }

  while (start <= end) {
    let t = String(pad(start))
    minutes.push({ value: t, name: config.format.replace('{value}', t) })
    start++
  }

  return minutes
}
